#include "TArtCalibSiStopper.hh" 
#include "TArtSiStopper.hh"
#include "TArtSiStopperPara.hh"
#include "TArtEURICAParameters.hh"
#include "TArtRawEventObject.hh"
#include "TArtCore.hh" 
#include "TArtStoreManager.hh"

#include <TROOT.h>
#include <TMath.h>
#include <TClonesArray.h>

//__________________________________________________________
TArtCalibSiStopper::TArtCalibSiStopper()
  :TArtReconstruction("EURICASiStopper")
{
  TArtCore::Info(__FILE__,"Creating the SiStopper detector objects...");

  sman = TArtStoreManager::Instance();
  setup = (TArtEURICAParameters*)sman->FindParameters("EURICAParameters");
  if(!setup){
    TArtCore::Info(__FILE__,"Can not find parameter handler: EURICAParameters");
    throw;
  }

  fSiStopperArray = new TClonesArray("TArtSiStopper",10);
  fSiStopperArray->SetName("EURICASiStopper");
  fSiStopperArray->SetOwner();
  sman->AddDataContainer(fSiStopperArray);

}

//__________________________________________________________
TArtCalibSiStopper::~TArtCalibSiStopper()  {
  ClearData();
}

//__________________________________________________________
void TArtCalibSiStopper::LoadData()   {

  TArtRawEventObject *fEvent = (TArtRawEventObject *)sman->FindDataContainer("RawEvent");

  for(int i=0;i<fEvent->GetNumSeg();i++){
    TArtRawSegmentObject *seg = fEvent->GetSegment(i);
    int device   = seg->GetDevice();
    int fpl      = seg->GetFP();
    int detector = seg->GetDetector();
    if((11==fpl) && (BETAA==detector || BETAT==detector || HIA==detector)) LoadData(seg);
  }
  return;

}

//__________________________________________________________
void TArtCalibSiStopper::LoadData(TArtRawSegmentObject *seg)   {

  int device   = seg->GetDevice();
  int fpl      = seg->GetFP();
  int detector = seg->GetDetector();
  for(int j=0;j<seg->GetNumData();j++){
    TArtRawDataObject *d = seg->GetData(j);
    int geo = d->GetGeo(); 
    int ch = d->GetCh(); 
    unsigned int val = d->GetVal();
    TArtRIDFMap mm(fpl,detector,geo,ch);
    TArtSiStopperPara *para = (TArtSiStopperPara*)setup->FindSiStopperPara(&mm);
    if(NULL == para){
      //TArtCore::Error(__FILE__,"Could not find TArtSiStopperPara...: Dev:%d, %s", device, mm.GetMapInfo());
      continue;
    }

    int id = para->GetID();
    int nsi = fSiStopperArray->GetEntries();
    TArtCore::Debug(__FILE__,"Entries in SiStopperArray: %d, SiStopper-ID:%d", nsi, id);
    TArtSiStopper * si = FindSiStopper(id);

    if(NULL==si){
      new ((*fSiStopperArray)[nsi]) TArtSiStopper();
      si = (TArtSiStopper *)fSiStopperArray->At(nsi);
      si->SetID(id);
      fSiStopperParaArray.push_back(para);
    }

    // set raw data
    // do not need to scan channel because of only 1 channel for 1 si.
    if(*(para->GetHGADCMap()) == mm)
      si->SetRawHGADC(val);
    if(*(para->GetLGADCMap()) == mm)
      si->SetRawLGADC(val);
    if(*(para->GetTDCMap()) == mm)
      si->SetRawTDC(val);

  }

  fDataLoaded = true;
  return;
}

//__________________________________________________________
void TArtCalibSiStopper::ClearData()   {
  fSiStopperArray->Delete();
  fSiStopperParaArray.clear();
  fDataLoaded = false;
  fReconstructed = false;
  return;
}

//__________________________________________________________
void TArtCalibSiStopper::ReconstructData()   { // call after the raw data are loaded

  if(!fDataLoaded) LoadData();

  for(int i=0;i<GetNumSiStopper();i++){
    TArtSiStopper *si = (TArtSiStopper*)fSiStopperArray->At(i);
    TArtSiStopperPara *para = fSiStopperParaArray[i];

    unsigned int hgadc = si->GetRawHGADC();
    unsigned int lgadc = si->GetRawLGADC();
    unsigned int tdc = si->GetRawTDC();

    // copy some information from para to data container
    si->SetID(para->GetID());
    si->SetDetectorName(*(para->GetDetectorName()));
    si->SetLocalChannel(para->GetLocalChannel());
    si->SetLocalLayer(para->GetLocalLayer());
    si->SetLayer(para->GetLayer());

    if(tdc>0 && tdc<1000000){ 
      Double_t fTime = tdc * para->GetTCal();
      si->SetTime(fTime);
    }
    if(hgadc>0 && hgadc<100000){ 
      Double_t fEnergy = (hgadc - para->GetHGQPed()) * para->GetHGQCal();
      si->SetHGEnergy(fEnergy);
    }
    if(lgadc>0 && lgadc<100000){ 
      Double_t fEnergy = (lgadc - para->GetLGQPed()) * para->GetLGQCal();
      si->SetLGEnergy(fEnergy);
    }

  }

  fReconstructed = true;
  return;
}

//__________________________________________________________
TArtSiStopper * TArtCalibSiStopper::GetSiStopper(int i) {
  return (TArtSiStopper *)fSiStopperArray->At(i);
}
//__________________________________________________________
TArtSiStopperPara * TArtCalibSiStopper::GetSiStopperPara(int i) {
  return (TArtSiStopperPara *)fSiStopperParaArray[i];
}
//__________________________________________________________
Int_t TArtCalibSiStopper::GetNumSiStopper() {
  return fSiStopperArray->GetEntries();
}
//__________________________________________________________
TArtSiStopper * TArtCalibSiStopper::FindSiStopper(Int_t id){
  for(int i=0;i<GetNumSiStopper();i++)
    if(id == ((TArtSiStopper*)fSiStopperArray->At(i))->GetID())
      return (TArtSiStopper*)fSiStopperArray->At(i);
  return NULL;
}

