#include "TArtCalibDCTrack.hh" 
#include "TArtDCTrack.hh"
#include "TArtDCHit.hh"
#include "TArtCore.hh" 
#include "TArtStoreManager.hh"

#include <TROOT.h>
#include <TFile.h>
#include <TMath.h>
#include <TMatrixD.h>
#include <TString.h>
#include <TClonesArray.h>

#include <iostream>
#include <stdlib.h>
#include <math.h>

//__________________________________________________________
TArtCalibDCTrack::TArtCalibDCTrack(const char *indata, 
				   const char *outdata, 
				   const char *lpattern)
  : TArtReconstruction(outdata), htdc(NULL), nlayer_x(0), nlayer_y(0), nlayer_u(0), nlayer_v(0)
{
  TArtCore::Info(__FILE__,"Creating the DCTrack objects...");
  sman = TArtStoreManager::Instance();

  fDCTrackArray = new TClonesArray("TArtDCTrack",10);
  fDCTrackArray->SetName(outdataname.Data());
  fDCTrackArray->SetOwner();
  sman->AddDataContainer(fDCTrackArray);

  nlayer = strlen(lpattern);
  TString lname;
  for(int i=0;i<nlayer;i++){
    lname = lpattern[i];
    if(lname == "X"){layer_map.push_back(nlayer_x); nlayer_x++;}
    else if(lname == "Y"){layer_map.push_back(nlayer_y); nlayer_y++;}
    else if(lname == "U"){layer_map.push_back(nlayer_u); nlayer_u++;}
    else if(lname == "V"){layer_map.push_back(nlayer_v); nlayer_v++;}
    else{TArtCore::Info(__FILE__,"Can not define corresponding wire-dir..: %s",lname.Data());}
  }

  hits_buffer   = new std::vector <TArtDCHit *> [nlayer];
  hits_buffer_x = new std::vector <TArtDCHit *> [nlayer_x];
  hits_buffer_y = new std::vector <TArtDCHit *> [nlayer_y];
  hits_buffer_u = new std::vector <TArtDCHit *> [nlayer_u];
  hits_buffer_v = new std::vector <TArtDCHit *> [nlayer_v];

  indataname = indata;

}

//__________________________________________________________
TArtCalibDCTrack::~TArtCalibDCTrack()  {
  ClearData();
  delete hits_buffer;
  delete hits_buffer_x;
  delete hits_buffer_y;
  delete hits_buffer_u;
  delete hits_buffer_v;

  if(htdc){
    delete htdc;
    htdc = NULL;
  }
}

//__________________________________________________________
void TArtCalibDCTrack::ClearData()   {
  for(int i=0;i<nlayer  ;i++) hits_buffer[i].clear();
  for(int i=0;i<nlayer_x;i++) hits_buffer_x[i].clear();
  for(int i=0;i<nlayer_y;i++) hits_buffer_y[i].clear();
  for(int i=0;i<nlayer_u;i++) hits_buffer_u[i].clear();
  for(int i=0;i<nlayer_v;i++) hits_buffer_v[i].clear();

  fDCTrackArray->Clear();
  fReconstructed = false;
  return;
}

//__________________________________________________________
void TArtCalibDCTrack::SetTDCDistribution(TH1 * inhtdc)   {
  char name[64];
  sprintf(name,"my%s",inhtdc->GetName());
  htdc = (TH1*)inhtdc->Clone(name);
  tdcwindow[0] = htdc->FindBin(200); 
  tdcwindow[1] = htdc->FindBin(1900); 
  tdcint = htdc->Integral(tdcwindow[0],tdcwindow[1]); 
}

//__________________________________________________________
void TArtCalibDCTrack::ReconstructData() { // call after the raw data are loaded
  // allocate each hit in hit_buffer
  SetHitsBuffer();

  // fast simple tracking
  //  RecoSimpleTrack(hits_buffer_x, nlayer_x, 0);
  //  RecoSimpleTrack(hits_buffer_y, nlayer_y, 1);

  // slow but accurate tracking
  RecoTrack();

  fReconstructed = true;
  return;
}

//__________________________________________________________
void TArtCalibDCTrack::SetHitsBuffer() {

  TClonesArray * hit_array = (TClonesArray *)sman->FindDataContainer(indataname.Data());
  Int_t num_hit = hit_array->GetEntries();

  // allocate each hit in hit_buffer
  for(int i=0;i<num_hit;i++){
    TArtDCHit * hit = (TArtDCHit *)hit_array->At(i);
    Int_t layer = hit->GetLayer();
    Double_t dl = CalcDriftLength(hit->GetTDC());
    if(fabs(dl)>200) continue; // too long dl, continue to next hit. 
    if(dl<0) continue; // negative dl, continue to next hit. 

    hits_buffer[layer].push_back(hit);
    if(*(hit->GetWireDirection()) == "X"){
      if(layer_map[layer]>=nlayer_x)
	TArtCore::Error(__FILE__,"layer-id(%d) is larger than expected num of layer-x %d",layer_map[layer],nlayer_x);
      hits_buffer_x[layer_map[layer]].push_back(hit);
    }
    else if(*(hit->GetWireDirection()) == "Y"){
      if(layer_map[layer]>=nlayer_y)
	TArtCore::Error(__FILE__,"layer-id(%d) is larger than expected num of layer-y %d",layer_map[layer],nlayer_y);
      hits_buffer_y[layer_map[layer]].push_back(hit);
    }
    else if(*(hit->GetWireDirection()) == "U"){
      if(layer_map[layer]>=nlayer_u)
	TArtCore::Error(__FILE__,"layer-id(%d) is larger than expected num of layer-u %d",layer_map[layer],nlayer_u);
      hits_buffer_u[layer_map[layer]].push_back(hit);
    }
    else if(*(hit->GetWireDirection()) == "V"){
      if(layer_map[layer]>=nlayer_v)
	TArtCore::Error(__FILE__,"layer-id(%d) is larger than expected num of layer-v %d",layer_map[layer],nlayer_v);
      hits_buffer_v[layer_map[layer]].push_back(hit);
    }
    else{TArtCore::Info(__FILE__,"Can not find corresponding wire-dir..: %s", hit->GetWireDirection()->Data());}
  } 

  return;

}

//__________________________________________________________
Double_t TArtCalibDCTrack::CalcDriftLength(Double_t tdc) {

  if(htdc){ // if tdc distribution is set, calculation by using tdc distribution
    Int_t hitbin = htdc->FindBin(tdc);
    if(hitbin>tdcwindow[1] || hitbin<tdcwindow[0]) return -9999;
    return pitch * 0.5 * (Double_t)htdc->Integral(hitbin,tdcwindow[1])/tdcint;
  }
  else if(driftv>0){ // if drift verocity is set, calculation on the assumtion of dl=(tdczero-tdc)*driftv
    Double_t dl = driftv * (tdczero-(Double_t)tdc);
    if(dl<0){return 0;}
    else if(dl<=pitch * 0.5 * TMath::Sqrt(2)){ 
      return dl;
    }
    return -9999;
  }
  else{
    TArtCore::Error(__FILE__,"no driftv nor htdc for dl calc");
    return -9999;
  }
}

//
// function to make track candidates. not select only one track.
//
//__________________________________________________________
void TArtCalibDCTrack::RecoSimpleTrack(std::vector <TArtDCHit *> *hits,Int_t nl, Int_t dir){

  Double_t lpos=0;
  if(nl>=4) // at least 4 layers are necessary
    for(int i=0;i<(int)hits[0].size();i++){
      for(int j=0;j<(int)hits[1].size();j++){
	//	TArtCore::Error(__FILE__,"xwire-pos01 %s %s (%f, %f)", hits_buffer_x[0][i]->GetDetectorName()->Data(), hits_buffer_x[1][j]->GetDetectorName()->Data(), hits_buffer_x[0][i]->GetWirePosition(), hits_buffer_x[1][j]->GetWirePosition());
	if(fabs(hits[0][i]->GetWirePosition() - hits[1][j]->GetWirePosition()) == pitch/2.){
	  Int_t sign = hits[0][i]->GetWirePosition() > hits[1][j]->GetWirePosition() ? -1 : 1;
	  Double_t pos[nl][2];

	  lpos = sign*CalcDriftLength(hits[0][i]->GetTDC());
	  pos[0][0]=hits[0][i]->GetWirePosition()+lpos;
	  hits[0][i]->SetPosition(lpos);
	  pos[0][1]=hits[0][i]->GetWireZPosition();

	  lpos = -sign*CalcDriftLength(hits[1][j]->GetTDC());
	  pos[1][0]=hits[1][j]->GetWirePosition()+lpos;
	  hits[1][j]->SetPosition(lpos);
	  pos[1][1]=hits[1][j]->GetWireZPosition();

	  for(int ii=0;ii<(int)hits[nl-2].size();ii++){
	    for(int jj=0;jj<(int)hits[nl-1].size();jj++){
	      //TArtCore::Error(__FILE__,"wire-pos23 %s, %s (%f, %f)", hits[nl-2][ii]->GetDetectorName()->Data(), hits[nl-1][jj]->GetDetectorName()->Data(), hits[nl-2][ii]->GetWirePosition(), hits[nl-1][jj]->GetWirePosition());
	      if(fabs(hits[nl-2][ii]->GetWirePosition() - hits[nl-1][jj]->GetWirePosition()) == pitch/2.){
		sign = hits[nl-2][ii]->GetWirePosition() > hits[nl-1][jj]->GetWirePosition() ? -1 : 1;

		lpos = sign*CalcDriftLength(hits[nl-2][ii]->GetTDC());
		pos[2][0]=hits[nl-2][ii]->GetWirePosition()+lpos;
		hits[nl-2][ii]->SetPosition(lpos);
		pos[2][1]=hits[nl-2][ii]->GetWireZPosition();

		lpos = -sign*CalcDriftLength(hits[nl-1][jj]->GetTDC());
		pos[3][0]=hits[nl-1][jj]->GetWirePosition()+lpos;
		hits[nl-1][jj]->SetPosition(lpos);
		pos[3][1]=hits[nl-1][jj]->GetWireZPosition();

		int nhit = fDCTrackArray->GetEntries();
		new ((*fDCTrackArray)[nhit]) TArtDCTrack();
		TArtDCTrack * atrack = (TArtDCTrack *)fDCTrackArray->At(nhit);
		// calculate first track only with first 2 and last 2 points.
		CalcSimpleTrack(pos,4,atrack,dir);

		if(nl>4){
		  for(int l=0;l<nl;l++)
		    FindAssociatePos(atrack,&(hits[l]),pos[l]);
		  // re-calculate track with all layer points.
		  CalcSimpleTrack(pos,nl,atrack,dir);
		}
	      } // end of if(fabs(hits_buffer_x[nl_x-2][ii]->GetWirePosition() - hits_buffer_x[nl_x-1][jj]->GetWirePosition()) == pitch/2.)
	    }
	  }
	} // end of if(fabs(hits_buffer_x[0][i]->GetWirePosition() - hits_buffer_x[1][j]->GetWirePosition()) == pitch/2.)
      }
    }

  return;
}

//__________________________________________________________
void TArtCalibDCTrack::CalcSimpleTrack(Double_t pos[][2], Int_t nl, TArtDCTrack *tr, Int_t dir) {

  //on the assumption of linear track, the track is reconstructed analytically.
  //X = a + b * Z;

  TMatrixD xvec(2,1); xvec.Zero();
  TMatrixD xmat(2,2); xmat.Zero();

  Int_t nhit = 0;
  for(int i=0;i<nl;i++) {
    //TArtCore::Error(__FILE__,"CalcTrack %d, x:%f, z:%f",i,pos[i][0],pos[i][1]);
    if(pos[i][0]>-9000){
      xvec(0,0) += pos[i][1] * pos[i][0]; // b(1) in rayfit
      xvec(1,0) += pos[i][0]; // b(2) in rayfit
      xmat(0,1) += pos[i][1]; // a(1,2) in rayfit
      xmat(1,0) += pos[i][1]; // a(1,2) in rayfit
      xmat(0,0) += pos[i][1] * pos[i][1];   // a(1,1) in rayfit
      xmat(1,1) ++;  // a(2,2) in rayfit
      nhit ++;
    }
  }

  Double_t track_x=-99999, track_a=-99999;
  Double_t chi2 = 9999;
  if(nhit>0){
    Double_t det = xmat(0,0) * xmat(1,1) - xmat(0,1) * xmat(1,0);
    if(fabs(det)>1e-5){
      TMatrixD rxvec = xmat.Invert() * xvec;
      track_x = rxvec(1,0);
      track_a = rxvec(0,0);
      chi2 = 0;
      for(int i=0;i<nl;i++)
	chi2 += pow(track_x+track_a*pos[i][1] - pos[i][0],2);
    }
  }

  tr->SetNumHitLayer(nhit);
  tr->SetChi2(chi2);
  tr->SetNDF(nl-2-1);
  tr->SetPosition(track_x,dir);
  tr->SetAngle(track_a,dir);

  return;
}

//__________________________________________________________
void TArtCalibDCTrack::FindAssociatePos(TArtDCTrack *tr, std::vector <TArtDCHit *> *hits, Double_t *pos) {

  pos[0] = -9999; pos[1] = -9999;
  for(int i=0;i<(int)hits->size();i++){
    Double_t dist = tr->GetPosition()+tr->GetAngle()*(*hits)[i]->GetWireZPosition() - (*hits)[i]->GetWirePosition();
    if(fabs(dist) <= pitch/2.){
      Int_t sign = dist>=0 ? 1 : -1;
      Double_t lpos = sign*CalcDriftLength((*hits)[i]->GetTDC());
      pos[0] = (*hits)[i]->GetWirePosition()+lpos;
      (*hits)[i]->SetPosition(lpos);
      pos[1] = (*hits)[i]->GetWireZPosition();
    }
  }
}

//
//
//
//__________________________________________________________
void TArtCalibDCTrack::RecoTrack() {

  Int_t ntrack_keep = 10; // number of tracks to keep for following analysis

  TObjArray tracks_x;
  TObjArray tracks_y;
  TObjArray tracks_u;
  TObjArray tracks_v;
  Int_t ntrx=0, ntry=0, ntru=0, ntrv=0;

  CalcTrack(hits_buffer_x, nlayer_x, &tracks_x);
  ntrx = tracks_x.GetEntriesFast();
  tracks_x.Sort();
  if(ntrx>ntrack_keep){
    tracks_x.RemoveRange(ntrack_keep,ntrx-1);
    ntrx = tracks_x.GetEntriesFast();
  }

  if(nlayer_y>0){
    CalcTrack(hits_buffer_y, nlayer_y, &tracks_y);
    ntry = tracks_y.GetEntriesFast();
    tracks_y.Sort();
    if(ntry>ntrack_keep){
      tracks_y.RemoveRange(ntrack_keep,ntry-1);
      ntry = tracks_y.GetEntriesFast();
    }
  }

  if(nlayer_u>0){
    CalcTrack(hits_buffer_u, nlayer_u, &tracks_u);
    ntru = tracks_u.GetEntriesFast();
    tracks_u.Sort();
    if(ntru>ntrack_keep){
      tracks_u.RemoveRange(ntrack_keep,ntru-1);
      ntru = tracks_u.GetEntriesFast();
    }
  }

  if(nlayer_v>0){
    CalcTrack(hits_buffer_v, nlayer_v, &tracks_v);
    ntrv = tracks_v.GetEntriesFast();
    tracks_v.Sort();
    if(ntrv>ntrack_keep){
      tracks_v.RemoveRange(ntrack_keep,ntrv-1);
      ntrv = tracks_v.GetEntriesFast();
    }
  }

  if( nlayer_u>0 || nlayer_v>0 ){ // FDC
   TObjArray mytracks; // track buffer before sorting
    for(int ix=0;ix<ntrx;ix++)
      for(int iu=0;iu<ntru;iu++)
	//for(int iu=0;iu<1;iu++)
	for(int iv=0;iv<ntrv;iv++){
	  //for(int iv=0;iv<1;iv++){
	  TArtDCTrack * mytr = (TArtDCTrack *)CalcTrackWith1DTracks((Track1D*)tracks_x[ix],(Track1D*)tracks_u[iu],(Track1D*)tracks_v[iv]);
	  if(mytr) mytracks.Add(mytr);
	}
    mytracks.Sort(mytracks.GetEntriesFast());
    for(int i=0;i<mytracks.GetEntriesFast()&&i<ntrack_keep;i++){ // contain up to ntrack_keep track candidates
      int ntr = fDCTrackArray->GetEntries();
      new ((*fDCTrackArray)[ntr]) TArtDCTrack(*((TArtDCTrack *)mytracks.At(i)));
      //((TArtDCTrack *)fDCTrackArray->At(ntr))->Print();
    }
    //std::cout << std::endl << std::endl;
    for(int i=0; i<= mytracks.GetLast(); i++) 
      if(mytracks[i]) delete mytracks[i];
  }
  else{ // BDC
    for(int i=0;i<ntrx;i++){
      int ntr = fDCTrackArray->GetEntries();
      new ((*fDCTrackArray)[ntr]) TArtDCTrack();
      TArtDCTrack * atrack = (TArtDCTrack *)fDCTrackArray->At(ntr);
      atrack->SetNumHitLayer(((Track1D*)tracks_x[i])->ndf + 2 + 1);
      atrack->SetChi2(((Track1D*)tracks_x[i])->chi2);
      atrack->SetNDF(((Track1D*)tracks_x[i])->ndf);
      atrack->SetPosition(((Track1D*)tracks_x[i])->x,0);
      atrack->SetAngle(((Track1D*)tracks_x[i])->a,0);
      //atrack->Print();
    }
    //std::cout << std::endl << std::endl;
    for(int i=0;i<ntry;i++){
      int ntr = fDCTrackArray->GetEntries();
      new ((*fDCTrackArray)[ntr]) TArtDCTrack();
      TArtDCTrack * atrack = (TArtDCTrack *)fDCTrackArray->At(ntr);
      atrack->SetNumHitLayer(((Track1D*)tracks_y[i])->ndf + 2 + 1);
      atrack->SetChi2(((Track1D*)tracks_y[i])->chi2);
      atrack->SetNDF(((Track1D*)tracks_y[i])->ndf);
      atrack->SetPosition(((Track1D*)tracks_y[i])->x,1);
      atrack->SetAngle(((Track1D*)tracks_y[i])->a,1);
    }
  }


  /*
  std::vector <Track1D *> tracks_u = CalcTrack(hits_buffer_u, nlayer_u);
  Int_t ntru = tracks_u.size();
  for(int i=0;i<ntru;i++){
    int ntr = fDCTrackArray->GetEntries();
    new ((*fDCTrackArray)[ntr]) TArtDCTrack();
    TArtDCTrack * atrack = (TArtDCTrack *)fDCTrackArray->At(ntr);
    atrack->SetNumHitLayer(tracks_u[i]->ndf + 2 + 1);
    atrack->SetChi2(tracks_u[i]->chi2);
    atrack->SetNDF(tracks_u[i]->ndf);
    atrack->SetPosition(tracks_u[i]->x,1);
    atrack->SetAngle(tracks_u[i]->a,1);
  }
  */

  for(int i=0; i<= tracks_x.GetLast(); i++) 
    if(tracks_x[i]) delete tracks_x[i];
  for(int i=0; i<= tracks_y.GetLast(); i++) 
    if(tracks_y[i]) delete tracks_y[i];
  for(int i=0; i<= tracks_u.GetLast(); i++) 
    if(tracks_u[i]) delete tracks_u[i];
  for(int i=0; i<= tracks_v.GetLast(); i++) 
    if(tracks_v[i]) delete tracks_v[i];

  return;
}

//
//
//
//__________________________________________________________
void TArtCalibDCTrack::CalcTrack(std::vector <TArtDCHit *> *hits, Int_t numlayer, TObjArray *tracks) {

  Int_t npattern = 1; 
  for(int i=0;i<numlayer;i++){
    if(hits[i].size()>0) npattern *= hits[i].size();
    //std::cout << hits[i].size() << " ";
  }
  //std::cout << npattern << std::endl << std::endl;
  if(npattern<0 || npattern>pow(10,numlayer)) return;

  Int_t np = 1; 
  std::vector <TArtDCHit *> *scanhits = new std::vector <TArtDCHit *> [npattern];
  for(int i=0;i<numlayer;i++){
    if(hits[i].size()>0){
      //std::cout << i << "(" << hits[i].size() << "/" << npattern << ") " ;
      for(int j=0;j<npattern;j+=np){
	for(int k=0;k<np;k++){
	  //std::cout << (j%(np*hits[i].size()))/np << " ";
	  scanhits[j+k].push_back(hits[i][(j%(np*hits[i].size()))/np]); 
	}
      }
      np *= hits[i].size(); 
      //std::cout << std::endl;
    }
  }

  for(int i=0;i<npattern;i++){
    Track1D *mytr = CalcTrack1D(&(scanhits[i]));
    if( mytr->chi2<9999. && mytr->ndf + 2 + 1 == numlayer){
	//for(int j=0;j<mytr.nhit;j++)std::cout << j << " " << mytr.hitx[j] << std::endl;
      tracks->Add(mytr);
    }
    scanhits[i].clear();
  }

  return;
}

//________________________________________________________________________
TArtCalibDCTrack::Track1D * TArtCalibDCTrack::CalcTrack1D(std::vector <TArtDCHit *> *hits) {

  Track1D *tr = new Track1D; 
  TMatrixD vec(2,1); 
  TMatrixD mat(2,2); 

  Int_t nl = hits->size();
  Int_t npattern = (Int_t)pow(2,nl);
  Double_t *xpos = new Double_t[nl];
  Double_t *zpos = new Double_t[nl];
  tr->hitx = new Double_t[nl];
  tr->hitz = new Double_t[nl];
  Double_t bestchi2 = 9999.;

  for(int i=0;i<npattern;i++){

    vec.Zero();
    mat.Zero();
    for(int l=0;l<nl;l++){
      Double_t dl = CalcDriftLength((*hits)[l]->GetTDC());
      xpos[l] = (*hits)[l]->GetWirePosition();
      zpos[l] = (*hits)[l]->GetWireZPosition();
      Int_t sign = i&((Int_t)pow(2,l)) ? 1 : -1;
      xpos[l] += sign*dl;

      vec(0,0) += xpos[l];
      vec(1,0) += zpos[l] * xpos[l];
      
      mat(0,0) ++;
      mat(0,1) += zpos[l];
      mat(1,1) += zpos[l] * zpos[l];

    }
    mat(1,0) = mat(0,1); 

    Double_t det = mat(0,0) * mat(1,1) - mat(0,1) * mat(1,0);
    if(fabs(det)>1e-5){
      TMatrixD rxvec = mat.Invert() * vec;
      Double_t chi2 = 0;
      for(int j=0;j<nl;j++)
	chi2 += pow(rxvec(0,0)+rxvec(1,0)*zpos[j] - xpos[j],2);

      if(chi2<bestchi2){
	bestchi2 = chi2;
	tr->x = rxvec(0,0);
	tr->a = rxvec(1,0);
	tr->chi2 = bestchi2;
	tr->ndf = nl-2-1;
	tr->nhit = nl;
	for(int j=0;j<nl;j++){
	  tr->hitx[j] = xpos[j];
	  tr->hitz[j] = zpos[j];
	}
      }
    }

  }

  delete xpos;
  delete zpos;

  /*
  TArtCore::Info(__FILE__,"x:%f a:%f chi2:%f ndf:%d",tr.x,tr.a,tr.chi2,tr.ndf);
  for(int i=0;i<nl;i++)
    TArtCore::Info(__FILE__,"i:%d, x:%f, z:%f",i,tr.hitx[i],tr.hitz[i]);
  TArtCore::Error(__FILE__,"outdataname:%s",outdataname.Data());
  */

  return tr;

}

//__________________________________________________________
TArtDCTrack * TArtCalibDCTrack::CalcTrackWith1DTracks(Track1D *trx, Track1D *tru, Track1D *trv) {

  TMatrixD rvec(4,1);
  TMatrixD vec(4,1); vec.Zero();
  TMatrixD mat(4,4); mat.Zero();

  Int_t nx=0, nu=0, nv=0;

  Double_t uangle = 30./180.*TMath::Pi();
  Double_t vangle = -30./180.*TMath::Pi();

  for(int ix=0;ix<trx->nhit;ix++) {
    nx ++;

    vec(0,0) += trx->hitx[ix];
    vec(1,0) += trx->hitz[ix] * trx->hitx[ix];
    
    mat(0,0) ++;
    mat(0,1) += trx->hitz[ix];
    mat(1,1) += trx->hitz[ix] * trx->hitz[ix];
  }
  /*
  for(int iy=0;iy<tr->nhit;iy++) {
      ny ++;

      vec(2,0) += x;
      vec(3,0) += z * x;

      mat(2,2) ++;
      mat(2,3) += z;
      mat(3,3) += z * z;
  }
  */
  for(int iu=0;iu<tru->nhit;iu++) { // + 30 degree
    nu ++;

    vec(0,0) +=                 tru->hitx[iu] * TMath::Cos(uangle);
    vec(1,0) += tru->hitz[iu] * tru->hitx[iu] * TMath::Cos(uangle);
    vec(2,0) +=                 tru->hitx[iu] * TMath::Sin(uangle);
    vec(3,0) += tru->hitz[iu] * tru->hitx[iu] * TMath::Sin(uangle);

    mat(0,0) +=                  1/2. + TMath::Cos(2*uangle)/2.;
    mat(0,1) += tru->hitz[iu] * (1/2. + TMath::Cos(2*uangle)/2.);
    mat(0,2) +=                 TMath::Sin(2*uangle)/2.;
    mat(0,3) += tru->hitz[iu] * TMath::Sin(2*uangle)/2.;
    mat(1,1) += TMath::Power(tru->hitz[iu],2) * (1/2. + TMath::Cos(2*uangle)/2.);
    mat(1,3) += TMath::Power(tru->hitz[iu],2) * TMath::Sin(2*uangle)/2.;
    mat(2,2) +=                  1/2. - TMath::Cos(2*uangle)/2.;
    mat(2,3) += tru->hitz[iu] * (1/2. - TMath::Cos(2*uangle)/2.);
    mat(3,3) += TMath::Power(tru->hitz[iu],2) * (1/2. - TMath::Cos(2*uangle)/2.);
  }
  for(int iv=0;iv<trv->nhit;iv++) { // - 30 degree
    nv ++;

    vec(0,0) +=                 trv->hitx[iv] * TMath::Cos(vangle);
    vec(1,0) += trv->hitz[iv] * trv->hitx[iv] * TMath::Cos(vangle);
    vec(2,0) +=                 trv->hitx[iv] * TMath::Sin(vangle);
    vec(3,0) += trv->hitz[iv] * trv->hitx[iv] * TMath::Sin(vangle);

    mat(0,0) +=                  1/2. + TMath::Cos(2*vangle)/2.;
    mat(0,1) += trv->hitz[iv] * (1/2. + TMath::Cos(2*vangle)/2.);
    mat(0,2) +=                 TMath::Sin(2*vangle)/2.;
    mat(0,3) += trv->hitz[iv] * TMath::Sin(2*vangle)/2.;
    mat(1,1) += TMath::Power(trv->hitz[iv],2) * (1/2. + TMath::Cos(2*vangle)/2.);
    mat(1,3) += TMath::Power(trv->hitz[iv],2) * TMath::Sin(2*vangle)/2.;
    mat(2,2) +=                  1/2. - TMath::Cos(2*vangle)/2.;
    mat(2,3) += trv->hitz[iv] * (1/2. - TMath::Cos(2*vangle)/2.);
    mat(3,3) += TMath::Power(trv->hitz[iv],2) * (1/2. - TMath::Cos(2*vangle)/2.);
  }

  mat(1,0) = mat(0,1); 
  mat(1,2) = mat(0,3);
  mat(2,0) = mat(0,2); 
  mat(2,1) = mat(1,2); 
  mat(3,0) = mat(0,3); 
  mat(3,1) = mat(1,3); 
  mat(3,2) = mat(2,3); 

  if(mat == 0) return NULL;

  Double_t chi2=0;
  //Double_t chi2x=0, chi2u=0, chi2v=0;
  Double_t det = mat.Determinant();
  if(fabs(det)>1e-5){
    rvec = mat.Invert() * vec;
    for(int i=0;i<nx;i++){
      chi2 += pow(rvec(0,0)+rvec(1,0)*trx->hitz[i] - trx->hitx[i],2);
      //chi2x += pow(rvec(0,0)+rvec(1,0)*trx->hitz[i] - trx->hitx[i],2);
      //if(i==0)TArtCore::Info(__FILE__,"dx:%f",chi2x);
    }
    for(int i=0;i<nu;i++){
      chi2 += pow((rvec(0,0)+rvec(1,0)*tru->hitz[i] - tru->hitx[i]*TMath::Cos(uangle))*TMath::Cos(uangle),2) 
	+ pow((rvec(2,0)+rvec(3,0)*tru->hitz[i] - tru->hitx[i]*TMath::Sin(uangle))*TMath::Sin(uangle),2);
      //chi2u += pow((rvec(0,0)+rvec(1,0)*tru->hitz[i] - tru->hitx[i]*TMath::Cos(uangle)),2) + pow((rvec(2,0)+rvec(3,0)*tru->hitz[i] - tru->hitx[i]*TMath::Sin(uangle)),2); 
      //      if(i==0)TArtCore::Info(__FILE__,"du:%f, uchi:%f",chi2u,pow((rvec(0,0)+rvec(1,0)*tru->hitz[i] - tru->hitx[i]*TMath::Cos(uangle))*TMath::Cos(uangle),2) + pow((rvec(2,0)+rvec(3,0)*tru->hitz[i] - tru->hitx[i]*TMath::Sin(uangle))*TMath::Sin(uangle),2));
    }
    for(int i=0;i<nv;i++){
      chi2 += pow((rvec(0,0)+rvec(1,0)*trv->hitz[i] - trv->hitx[i]*TMath::Cos(vangle))*TMath::Cos(vangle),2) 
	+ pow((rvec(2,0)+rvec(3,0)*trv->hitz[i] - trv->hitx[i]*TMath::Sin(vangle))*TMath::Sin(vangle),2);
      //chi2v += pow((rvec(0,0)+rvec(1,0)*trv->hitz[i] - trv->hitx[i]*TMath::Cos(vangle))*TMath::Cos(vangle),2) + pow((rvec(2,0)+rvec(3,0)*trv->hitz[i] - trv->hitx[i]*TMath::Sin(vangle))*TMath::Sin(vangle),2);
    }
  }

  //TArtCore::Info(__FILE__,"chi2x:%f, chi2u: %f, chi2v: %f",chi2x,chi2u,chi2v);

  TArtDCTrack *tr = new TArtDCTrack;
  tr->SetNumHitLayer(nx+nu+nv);
  tr->SetChi2(chi2);
  tr->SetNDF(nx+nu+nv -2 -1);
  tr->SetPosition(rvec(0,0));
  tr->SetAngle(rvec(1,0));
  tr->SetPosition(rvec(2,0),1);
  tr->SetAngle(rvec(3,0),1);

  return tr;
}

//__________________________________________________________
TArtDCTrack * TArtCalibDCTrack::GetDCTrack(int i) {
  return (TArtDCTrack *)fDCTrackArray->At(i);
}
//__________________________________________________________
Int_t TArtCalibDCTrack::GetNumDCTrack() {
  return fDCTrackArray->GetEntries();
}
//__________________________________________________________
Int_t TArtCalibDCTrack::GetNumDCHit(int l) {
  return hits_buffer[l].size();
}

//________________________________________________________________________
TArtCalibDCTrack::Track1D& TArtCalibDCTrack::Track1D::operator=(const TArtCalibDCTrack::Track1D &right){
  // overloading the copy operator

  if(this != &right){
    x = right.x;
    a = right.a;
    chi2 = right.chi2;
    ndf = right.ndf;
    nhit = right.nhit;
    hitx = new Double_t [nhit];
    hitz = new Double_t [nhit];
    for(int i=0;i<nhit;i++){
      hitx[i] = right.hitx[i];
      hitz[i] = right.hitz[i];
    }
  }
  return *this;
}

ClassImp(TArtCalibDCTrack)

