
#include "TArtCalibNEBULA.hh" 

#include <cmath>

//__________________________________________________________
TArtCalibNEBULA::TArtCalibNEBULA()
  : TArtReconstruction("NEBULAPla"), fSAMURAIParameters(0)
{
  TArtCore::Info(__FILE__,"Creating the NEBULA detector objects...");

  fStoreManager = TArtStoreManager::Instance();
  fNEBULAPlaArray = new TArtNEBULAPlaArray("TArtNEBULAPla", 300);
  fNEBULAPlaArray->SetOwner();
  fNEBULAPlaArray->SetName("NEBULAPla");
  fNEBULAPlaParaArray = new TArtNEBULAPlaParaArray("TArtNEBULAPlaPara", 300);
  fNEBULAPlaParaArray->SetOwner();
  fNEBULAPlaParaArray->SetName("NEBULAPlaPara");
  fStoreManager->AddDataContainer(fNEBULAPlaArray);
  fSAMURAIParameters = (const TArtSAMURAIParameters*)fStoreManager->FindParameters("SAMURAIParameters");
  if(!fSAMURAIParameters){
    TArtCore::Info(__FILE__,"Could not find SAMURAIParameters.");
    throw;
  }
}

//__________________________________________________________
TArtCalibNEBULA::~TArtCalibNEBULA()
{
  delete fNEBULAPlaParaArray;
  delete fNEBULAPlaArray;
}

//__________________________________________________________
void TArtCalibNEBULA::LoadRawData()
{
  TArtRawEventObject* fEvent = (TArtRawEventObject*)fStoreManager->FindDataContainer("RawEvent");

  for(int i=0; i<fEvent->GetNumSeg(); ++i){
    TArtRawSegmentObject* seg = fEvent->GetSegment(i);
    int device = seg->GetDevice();
    int fpl = seg->GetFP();
    int detector = seg->GetDetector();
    if(device != SAMURAI) continue;
    if(detector != NEBULA1Q &&
       detector != NEBULA1T &&
       detector != NEBULA2Q &&
       detector != NEBULA2T &&
       detector != NEBULA3Q &&
       detector != NEBULA3T &&
       detector != NEBULA4Q &&
       detector != NEBULA4T) continue; // analysis only tdc and qdc

    for(int j=0; j<seg->GetNumData(); ++j){
      TArtRawDataObject* d = seg->GetData(j);
      int geo = d->GetGeo(); 
      int ch = d->GetCh(); 
      unsigned int val = d->GetVal();
      TArtRIDFMap mm(fpl,detector,geo,ch);
      const TArtNEBULAPlaPara* parau = fSAMURAIParameters->FindNEBULAPlaUPara(mm);
      const TArtNEBULAPlaPara* parad = fSAMURAIParameters->FindNEBULAPlaDPara(mm);
      const TArtNEBULAPlaPara* para;
      if(parau && !parad){
	para = parau;
      }else if(!parau && parad){
	para = parad;
      }else if(parau && parad){
	TArtCore::Info(__FILE__,"TArtNEBULAPara...: Dev:%d, %s found in U and D",
		       device, mm.GetMapInfo());
	continue;
      }else{
	TArtCore::Info(__FILE__,"Could not find TArtNEBULAPlaPara...: Dev:%d, %s",
		       device, mm.GetMapInfo());
	continue;
      }
      if(para->GetIgnore()){
	TArtCore::Debug(__FILE__,"Ignore TArtNEBULAPlaPara...: Dev:%d, %s", device, mm.GetMapInfo());
	continue;
      }

      if(!FindNEBULAPlaPara(para->GetID())){
	int npara = fNEBULAPlaParaArray->GetEntries();
	new ((*fNEBULAPlaParaArray)[npara]) TArtNEBULAPlaPara(*para);

	fIDNPlaParaMap.insert(std::pair<int, int>(para->GetID(), npara));
      }
      TArtNEBULAPla* pla = FindNEBULAPla(para->GetID());
      if(!pla){
	int npla = fNEBULAPlaArray->GetEntries();
	pla = new ((*fNEBULAPlaArray)[npla]) TArtNEBULAPla();
	pla->SetID(para->GetID());
	pla->SetFpl(para->GetFpl());
	//	pla->SetDetectorName(para->GetDetectorName());
	pla->SetDetectorName(*(para->GetDetectorName()));
	pla->SetLayer(para->GetLayer());
	pla->SetSubLayer(para->GetSubLayer());
	pla->SetDetPos(para->GetDetPos());

	fIDNPlaMap.insert(std::pair<int, int>(para->GetID(), npla));
      }

      if(detector == NEBULA1T ||
         detector == NEBULA2T ||
         detector == NEBULA3T ||
         detector == NEBULA4T){
        if(parau){
          pla->SetTURaw(val);
        }else{
          pla->SetTDRaw(val);
        }
      }
      if(detector == NEBULA1Q ||
         detector == NEBULA2Q ||
         detector == NEBULA3Q ||
         detector == NEBULA4Q){
        if(parau){
          pla->SetQURaw(val);
        }else{
          pla->SetQDRaw(val);
	}
      }
    }
  }
  fDataLoaded = true;
}

//__________________________________________________________
void TArtCalibNEBULA::ReconstructData()
{

  if(!fDataLoaded) LoadRawData();
  //共通のoffset。どう値を取ってくるかは後で考える(とりあえず手打ち？)。
  Double_t posxoff = 0;
  Double_t posyoff = 0;
  Double_t poszoff = 200; //10m+2m

  for(int i=0;i<GetNumNEBULAPla();++i){
    TArtNEBULAPla* pla = GetNEBULAPla(i);
    Int_t id = pla->GetID();
    const TArtNEBULAPlaPara* para = FindNEBULAPlaPara(id);
    if(!para){
      TArtCore::Info(__FILE__,"cannot find para %d", id); // programming problem
      continue;
    }
    int turaw = pla->GetTURaw();
    int tdraw = pla->GetTDRaw();
    int quraw = pla->GetQURaw();
    int qdraw = pla->GetQDRaw();

    int hit = 0;
    if(quraw>0 && quraw<=4096) hit += 1;
    if(qdraw>0 && qdraw<=4096) hit += 2;
    if(turaw>0 && turaw<=4096) hit += 4;
    if(tdraw>0 && tdraw<=4096) hit += 8;      

    Double_t quped = quraw - para->GetQUPed();
    Double_t qdped = qdraw - para->GetQDPed();
    Double_t qucal = quped * para->GetQUCal();
    Double_t qdcal = qdped * para->GetQDCal();
    Double_t qaveped = -10000;
    Double_t qavecal = -10000;
    Double_t logqped = -10000;
    Double_t logqcal = -10000;
    Double_t ivsqrtquped = -10000; 
    Double_t ivsqrtqdped = -10000; 
    Double_t ivsqrtqaveped = -10000; 
    if(quped > 0 && qdped > 0){
      qaveped = sqrt(quped*qdped);
      logqped = log(quped/qdped);
      ivsqrtquped = 1/sqrt(quped);
      ivsqrtqdped = 1/sqrt(qdped);
      ivsqrtqaveped = 1/sqrt(qaveped);
    }
    if(qucal > 0 && qdcal > 0){    
      qavecal = sqrt(qucal*qdcal);
      logqcal = log(qucal/qdcal);
    }

    Double_t tucal = turaw * para->GetTUCal() + para->GetTUOff();
    Double_t tdcal = tdraw * para->GetTDCal() + para->GetTDOff();
    Double_t tuslw = -10000;
    Double_t tdslw = -10000;
    if(quped > 0) tuslw = tucal - para->GetTUSlw()/sqrt(quped);
    if(qdped > 0) tdslw = tdcal - para->GetTDSlw()/sqrt(qdped);
    Double_t dtraw = -10000;
    Double_t dtcal = -10000;
    Double_t dtslw = -10000;
    Double_t taveraw = -10000;
    Double_t tavecal = -10000;
    Double_t taveslw = -10000;
    Double_t poscal = -10000;
    Double_t posslw = -10000;
    if(turaw > 0 && tdraw > 0){
      dtraw = tdraw - turaw;
      dtcal = tdcal - tucal;
      dtslw = tdslw - tuslw;
      taveraw = (turaw + tdraw)/2.;
      tavecal = (tucal + tdcal)/2 + para->GetTAveOff();
      taveslw = (tuslw + tdslw)/2 + para->GetTAveOff();
      poscal = dtcal * para->GetDTCal() + para->GetDTOff();
      posslw = dtslw * para->GetDTCal() + para->GetDTOff();
    }

    Double_t pos[3];
    pos[0] = para->GetDetPos(0) + posxoff;
    if(turaw > 0 && tdraw > 0){
      pos[1] = posslw + para->GetDetPos(1) + posyoff; 
    }else{
      pos[1] = -10000;
    }
    pos[2] = para->GetDetPos(2) + poszoff;

    Double_t flightlength = -10000;
    Double_t ttofgamma = -10000;
    Double_t ttofneutron = -10000;
    if(turaw > 0 && tdraw > 0){
      flightlength = sqrt(pos[0]*pos[0]+pos[1]*pos[1]+pos[2]*pos[2]);
      ttofgamma = taveslw - flightlength/30.; //beta=1
      //      std::cout << taveslw << " " << flightlength/30. << std::endl;
      ttofneutron = taveslw - flightlength/20.; //beta=2/3
    }

    pla->SetHit(hit);
    pla->SetQUPed(quped);
    pla->SetQDPed(qdped);
    pla->SetQUCal(qucal);
    pla->SetQDCal(qdcal);
    pla->SetQAvePed(qaveped);
    pla->SetQAveCal(qavecal);
    pla->SetLogQPed(logqped);
    pla->SetLogQCal(logqcal);
    pla->SetIvSqrtQUPed(ivsqrtquped);
    pla->SetIvSqrtQDPed(ivsqrtqdped);
    pla->SetIvSqrtQAvePed(ivsqrtqaveped);

    pla->SetTUCal(tucal);
    pla->SetTDCal(tdcal);
    pla->SetTUSlw(tuslw);
    pla->SetTDSlw(tdslw);
    pla->SetDTRaw(dtraw);
    pla->SetDTCal(dtcal);
    pla->SetDTSlw(dtslw);    
    pla->SetTAveRaw(taveraw);
    pla->SetTAveCal(tavecal);
    pla->SetTAveSlw(taveslw);    
    pla->SetTTOFGamma(ttofgamma);
    pla->SetTTOFNeutron(ttofneutron);

    pla->SetPosCal(poscal);
    pla->SetPosSlw(posslw);
    pla->SetPos(pos);
    pla->SetFlightLength(flightlength);

    pla->SetDataState(1);
  }
  
  fReconstructed = true;
}

//__________________________________________________________
void TArtCalibNEBULA::CalMultiplicity()
{
  for(int i=0;i<GetNumNEBULAPla();++i){
    TArtNEBULAPla* pla = GetNEBULAPla(i);
    Int_t id = pla->GetID();
    const TArtNEBULAPlaPara* para = FindNEBULAPlaPara(id);
    if(!para){
      TArtCore::Info(__FILE__,"cannot find para %d", id); // programming problem
      continue;
    }

    if(!para->GetSubLayer() == 0) ++fMult.fMultiplicity_N; // mul. of all NEUT

    if(para->GetLayer() == 1){
      if(!para->GetSubLayer() == 0){
	++fMult.fMultiplicity_N1;      
	if(para->GetSubLayer() == 1){
	  ++fMult.fMultiplicity_N11;	  
	}else if(para->GetSubLayer() == 2){
	  ++fMult.fMultiplicity_N12;
	}
      }else if(para->GetSubLayer() == 0){
	++fMult.fMultiplicity_V1;
      }      
    }else if(para->GetLayer() == 2){
      if(!para->GetSubLayer() == 0){
	++fMult.fMultiplicity_N2;
	if(para->GetSubLayer() == 1){
	  ++fMult.fMultiplicity_N21;	  
	}else if(para->GetSubLayer() == 2){
	  ++fMult.fMultiplicity_N22;
	}
      }else if(para->GetSubLayer() == 0){
	++fMult.fMultiplicity_V2;
      }
    }
  }
}

//__________________________________________________________
void TArtCalibNEBULA::CalNeutron()
{
  if(GetNumNEBULAPla() != 1) return;

  TArtNEBULAPla* pla = GetNEBULAPla(0);
  fNeutron.fQ = pla->GetQAveCal();
  fNeutron.fT = pla->GetTAveSlw();

  double mn = 939;
  for(int i=0; i<3; ++i){
    fNeutron.fBeta[i] = std::abs(pla->GetPos(i)/fNeutron.fT/29.979);
    //    std::cout << pla->GetPos(i) << " " << fNeutron.fT << std::endl;
    if(fNeutron.fBeta[i] <= 1){ 
      fNeutron.fGamma[i] = 1/sqrt(1-fNeutron.fBeta[i]*fNeutron.fBeta[i]);
    }
    fNeutron.fP[i] = mn*fNeutron.fGamma[i]*fNeutron.fBeta[i];
  }
  fNeutron.fPabs = sqrt(fNeutron.fP[0]*fNeutron.fP[0]+
			fNeutron.fP[1]*fNeutron.fP[1]+
			fNeutron.fP[2]*fNeutron.fP[2]);

  fNeutron.fE = sqrt(mn*mn+fNeutron.fPabs*fNeutron.fPabs) - mn;
  fNeutron.fTheta = atan(sqrt(fNeutron.fP[0]*fNeutron.fP[0]
			      +fNeutron.fP[1]*fNeutron.fP[2])/fNeutron.fP[2]);
  fNeutron.fPhi = atan(fNeutron.fP[1]/fNeutron.fP[0]);


//  std::cout << fNeutron.fBeta[0] << " "
//	    << fNeutron.fBeta[1] << " "
//	    << fNeutron.fBeta[2] << " "
//	    << fNeutron.fPabs << " "
//	    << fNeutron.fE << " "
//	    << fNeutron.fTheta << " "
//	    << std::endl;

}

//__________________________________________________________
Int_t TArtCalibNEBULA::GetNumNEBULAPla() const
{
  return fNEBULAPlaArray->GetEntries();
}

//__________________________________________________________
TArtNEBULAPla* TArtCalibNEBULA::GetNEBULAPla(int i) const
{
  return (TArtNEBULAPla*)fNEBULAPlaArray->At(i);
}

//__________________________________________________________
TArtNEBULAPla* TArtCalibNEBULA::FindNEBULAPla(Int_t id) const
{
  std::map<int,int>::const_iterator it = fIDNPlaMap.find(id);
  if(it != fIDNPlaMap.end()) return GetNEBULAPla(it->second);
  else return 0;
}

//__________________________________________________________
TArtNEBULAPlaArray* TArtCalibNEBULA::GetNEBULAPlaArray() const
{
  return fNEBULAPlaArray;
}

//__________________________________________________________
Int_t TArtCalibNEBULA::GetNumNEBULAPlaPara() const
{
  return fNEBULAPlaParaArray->GetEntries();
}

//__________________________________________________________
TArtNEBULAPlaPara* TArtCalibNEBULA::GetNEBULAPlaPara(int i) const
{
  return (TArtNEBULAPlaPara*)fNEBULAPlaParaArray->At(i);
}

//__________________________________________________________
TArtNEBULAPlaPara* TArtCalibNEBULA::FindNEBULAPlaPara(int id) const
{
  std::map<int,int>::const_iterator it = fIDNPlaParaMap.find(id);
  if(it != fIDNPlaParaMap.end()) return GetNEBULAPlaPara(it->second);
  else return 0;
}

//__________________________________________________________
TArtNEBULAPlaParaArray* TArtCalibNEBULA::GetNEBULAPlaParaArray() const
{
  return fNEBULAPlaParaArray;
}

//__________________________________________________________
void TArtCalibNEBULA::ClearData()
{
  fMult.ClearData();
  fNeutron.ClearData();

  fNEBULAPlaArray->Delete();
  fNEBULAPlaParaArray->Delete();
  fIDNPlaMap.clear();
  fIDNPlaParaMap.clear();
  fDataLoaded = false;
  fReconstructed = false;
}

//__________________________________________________________
void TArtCalibNEBULANeutron::ClearData()
{
  fQ = -10000;
  fT = -10000;
  fP[0] = -10000;
  fP[1] = -10000;
  fP[2] = -10000;
  fPabs = -10000;
  fE = -10000;
  fTheta = -10000;
  fPhi = -10000;
}

//__________________________________________________________
void TArtCalibNEBULAMult::ClearData()
{
  fMultiplicity_N = 0;
  fMultiplicity_N1 = 0;
  fMultiplicity_N2 = 0;
  fMultiplicity_V1 = 0;
  fMultiplicity_V2 = 0;
  fMultiplicity_N11 = 0;
  fMultiplicity_N12 = 0;
  fMultiplicity_N21 = 0;
  fMultiplicity_N22 = 0;
}

//__________________________________________________________
TArtCalibNEBULANeutron::TArtCalibNEBULANeutron()
{
  ClearData();
}

//__________________________________________________________
TArtCalibNEBULANeutron::~TArtCalibNEBULANeutron()
{;}

//__________________________________________________________
TArtCalibNEBULAMult::TArtCalibNEBULAMult()
{
  ClearData();
}

//__________________________________________________________
TArtCalibNEBULAMult::~TArtCalibNEBULAMult()
{;}

//
ClassImp(TArtCalibNEBULANeutron)
ClassImp(TArtCalibNEBULAMult)
